/* 
   Authenticator.m

   Class to allow loginpanel to authenticate users

   Copyright (C) 2000 Free Software Foundation, Inc.

   Author:  Gregory John Casamento <borgheron@yahoo.com>
   Date: 2000
   
   This file is part of GNUstep.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Authenticator.m created by me on Wed 17-Nov-1999 */

#import "Authenticator.h"

@implementation Authenticator
// Initialization methods
- init
{
  [super init];
  username = nil;
  password = nil;
  pw = 0;
  
  return self;
}

- initWithUsername: (NSString *)user
          password: (NSString *)pass
{
  [self init];
  username = [user copy];
  password = [pass copy];
  return self;
}

// Accessor methods
- (void)setUsername: (NSString *)user
{
  username = user;
}

- (void)setPassword: (NSString *)pass
{
  password = pass;
}

- (NSString *)username
{
  return username;
}

- (NSString *)password
{
  return password;
}

// Action methods. 
- (BOOL)isPasswordCorrect
{
  BOOL result = NO;
  char *encrypted = 0,
    *correct = 0,
    *pword = 0;
#ifdef SHADOW_PASSWD
  struct spwd *sp;
#endif
  
  pword = (char *)[password cString];
  pw = getpwnam([username cString]);
  endpwent();
  if (!pw) 
    {
      return NO;
    }
  
#ifdef SHADOW_PASSWD
  sp = getspnam(pw->pw_name);
  endspent();
  if(sp) 
    {
      correct = sp->sp_pwdp;
    }
  else
#endif
    correct = pw->pw_passwd;
  
  if (correct == 0 || correct[0] == '\0') 
    {
      return YES;
    }

  encrypted = crypt(pword, correct);
  memset(pword, 0, strlen(pword));
  
  if (!strcmp(encrypted, correct)) 
    {
      return YES;
    } 
  else 
    {
      return NO;
    }

  return result;
}

- (void)setEnvironment
{
  char*          mail;
  
  // Check to see if the pw ivar is set.
  if (!pw) 
    {
      NSLog(@"User %@ not authenticated.", username);
      exit(0);
    }

#ifndef GNUSTEP  
  /* Set environment */
  environ = malloc(sizeof(char*) * 2);
  environ[0] = 0;
#else
  setenv("TERM", "vt100", 0);  // TERM=linux?
  setenv("HOME", pw->pw_dir, 1);
  setenv("SHELL", pw->pw_shell, 1);
  setenv("USER", pw->pw_name, 1);
  setenv("LOGNAME", pw->pw_name, 1);
  setenv("DISPLAY", ":0.0", 1);
  /* 
  mail = alloc(strlen(_PATH_MAILDIR) + strlen(pw->pw_name) + 2);
  strcpy(mail, _PATH_MAILDIR);
  strcat(mail, "/");
  strcat(mail, pw->pw_name);
  setenv("MAIL", mail, 1); 
  */
#endif

  chdir(pw->pw_dir);
}

- (void)switchUser
{
  // If pw ivar is not set, send a message to the log.
  if (!pw) 
    {
      NSLog(@"User %@ is not authenticated.", username);
      exit(0);
    }
  
#ifndef DEBUG
  // Set user and group ids
  if ((initgroups(pw->pw_name, pw->pw_gid) != 0) 
      || (setgid(pw->pw_gid) != 0) 
      || (setuid(pw->pw_uid) != 0)) 
    {
      NSLog(@"Could not switch to user id %@.", username);
      exit(0);
    }
#endif
  
  /* Execute login command */
  //execute(pw, path, script);
  //NSLog("could not execute login command");
  //exit(0);
}

- (void)executeCommand: (NSString *)command path: (NSString *)path
{
  char* args[4];
  NSString *shell = nil, *shell_basename = nil,
    *cmd = nil,    *temp = @"exec ",
    *arg = nil;
  
  // Extract the shell which should be run from the
  // user's GECOS information and execute the login
  // script.
  shell = [NSString stringWithCString: pw->pw_shell];
  shell_basename = [shell lastPathComponent];
  cmd = [path stringByAppendingPathComponent: command];
  arg = [temp stringByAppendingString: cmd];
  
  // Build the argument list to be sent to execv
  args[0] = malloc(strlen([shell_basename cString]) + 2);
  strcpy(args[0], "-");
  strcat(args[0], [shell_basename cString]);
  args[1] = "-c";
  args[2] = (char *)[arg cString];
  args[3] = 0;
  
#ifdef DEBUG
  printf("Would execute: %s %s %s %s %s\n",
	 [shell cString], args[0],
	 args[1], args[2], args[3] );
#else
  execv(shell, args);
#endif
}
@end
