/* 
   LoginPanelController.m

   Controller class which handles all activity in the loginpanel.

   Copyright (C) 2000 Free Software Foundation, Inc.

   Author:  Gregory John Casamento <borgheron@yahoo.com>
   Date: 2000
   
   This file is part of GNUstep.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "LoginPanelController.h"
#import "LoginImageView.h"
#import "Authenticator.h"


@implementation LoginPanelController
- init
{
  [super init];
  authenticator = [[Authenticator alloc] init];
  defaults = [NSUserDefaults standardUserDefaults];
  return self;
}

- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
  puts("Ordering front...");
  [window makeKeyAndOrderFront: self];
}

- (void)passwordEntered:(id)sender
{
  char *pwstring = 0;
  BOOL verified = NO;
  
  pwstring = (char *)[[passwordField stringValue] cString];
  printf("Password: %s\n", pwstring );
  printf("Verifying login...\n");
  [authenticator setUsername: [usernameField stringValue]];
  [authenticator setPassword: [passwordField stringValue]];
  verified = [authenticator isPasswordCorrect];
  if( verified )
    {
      puts("Proceed to login...");
      [self logUserIn];
    }
  else
    {
      puts("Verification failed...");
      [self rejectEntries];
    }
}

- (void)powerButton:(id)sender
{
  puts("Powering down...");
}

- (void)restartButton:(id)sender
{
  puts("Restarting computer...");
}

- (void)usernameEntered:(id)sender
{
  printf("Username: %s\n", [[usernameField stringValue] cString]);
  if( ![[usernameField stringValue] isEqualToString: [NSString string]] )
    {
      [passwordField becomeFirstResponder];
    }
  else
    {
      puts("Null usernames are illegal...");
      [self rejectEntries];
    }
}

- (void)awakeFromNib
{
  puts("The sleeper has awakened");
  [window initializeInterface];
}

- (void)rejectEntries
{
  [usernameField setStringValue: @""];
  [passwordField setStringValue: @""];
  [window waggle];
  [usernameField becomeFirstResponder];
}

- (void)logUserIn
{
  [window shrink];
  [window close];
  [self startUserSession];  // We will spend a great deal of time here!!
  [NSBundle loadNibNamed: @"NEXTSTEP_loginpanel"
	    owner: self];
}

- (void)startUserSession
{
  puts("La la la... user is doing what he/she has to do... done.");
  [authenticator switchUser];
  [authenticator setEnvironment];
  [authenticator executeCommand: @"loginscript" path: @"/usr/sbin"];
  puts("User is done...");
}

- (void)showInfo: (id)sender
{
  // load info panel
  if( infoPanel == nil )
    {
      if( ![NSBundle loadNibNamed: @"InfoPanel"
		     owner: self])
	{
	  puts("Problem loading info panel...");
	}
    }
  [infoPanel makeKeyAndOrderFront: self];
}
@end
